@@ -1,5 +1,6 @@
 import asyncio
 import dataclasses
+import inspect
 import mimetypes
 from pathlib import Path
 import threading
@@ -40,7 +41,7 @@ def has_time_passed(last_time, seconds: int = 5) -> bool:
 
 
 class BotClient:
-    BASE_URL = "https://botapi.rubika.ir/v3/{}/"
+    BASE_URL = "https://messengerg2b1.iranlms.ir/v3/{}/"
 
     def __init__(
             self,
@@ -54,10 +55,13 @@ class BotClient:
         self.token = token
         self.base_url = self.BASE_URL.format(token)
         self.handlers: Dict[str, List[Tuple[Tuple[Filter, ...], Callable]]] = {}
+        self.start_handlers: list[Callable] = []
+        self.shutdown_handlers: list[Callable] = []
+        self.middlewares: list[Callable] = []
         self.session = None
         self.running = False
         self.next_offset_id = None
-        self.processed_messages = deque(maxlen=10000)
+        self.processed_messages = deque(maxlen=500)
         self.rate_limit = rate_limit
         self.last_request_time = 0
         self.first_get_updates = True
@@ -66,6 +70,20 @@ class BotClient:
         self.connector = connector
         logger.info("Rubika client initialized, use_webhook=%s", use_webhook)
 
+    def on_start(self):
+        """
+        Decorator to register startup handlers.
+        
+        Example:
+            @bot.on_start()
+            async def hello(bot):
+                print("Bot started!")
+        """
+        def decorator(func: Callable):
+            self.start_handlers.append(func)
+            return func
+        return decorator
+
     async def __aenter__(self) -> "BotClient":
         await self.start()
         return self
@@ -92,13 +110,60 @@ class BotClient:
             connector = self.connector or aiohttp.TCPConnector(limit=50, limit_per_host=10)
             self.session = aiohttp.ClientSession(timeout=timeout, connector=connector)
         self.running = True
+
+        for handler in self.start_handlers:
+            if inspect.iscoroutinefunction(handler):
+                await handler(self)
+            else:
+                handler(self)
+
         logger.info("RubikaPY bot client started")
 
     async def stop(self):
         if self.session and not self.session.closed:
             await self.session.close()
         self.running = False
+
+        for handler in self.shutdown_handlers:
+            if inspect.iscoroutinefunction(handler):
+                await handler(self)
+            else:
+                handler(self)
+
         logger.info("RubikaPY bot client stopped")
+    
+    def on_shutdown(self):
+        """
+        Decorator to register shutdown handlers.
+        
+        Example:
+            @bot.on_shutdown()
+            async def bye(bot):
+                print("Bot stopped!")
+        """
+        def decorator(func: Callable):
+            self.shutdown_handlers.append(func)
+            return func
+        return decorator
+    
+    def middleware(self):
+        """
+        Decorator to register a middleware function.
+
+        Middleware receives (bot, update, next_middleware).
+        It must call `await next_middleware()` to continue chain.
+
+        Example:
+            @bot.middleware()
+            async def logger(bot, update, call_next):
+                print("🔹 Update received:", update)
+                await call_next()
+        """
+        def decorator(func: Callable):
+            self.middlewares.append(func)
+            logger.info(f"Middleware {func.__name__} registered")
+            return func
+        return decorator
 
     async def _make_request(self, method: str, data: Dict) -> Dict:
         await self._rate_limit_delay()
@@ -112,7 +177,6 @@ class BotClient:
             logger.debug(f"API response for {method}: {result}")
 
             if result["status"] != "OK":
-                print(result)
                 raise APIException(
                     status=result["status"],
                     dev_message=result.get("dev_message")
@@ -199,6 +263,82 @@ class BotClient:
             upload_url = await self.request_send_file(type)
             file_id = await self.upload_file(upload_url, file_name, file)
 
+        payload = {
+            "chat_id": chat_id,
+            "file_id": file_id,
+            "text": text,
+            "disable_notification": disable_notification,
+            "chat_keypad_type": chat_keypad_type.value,
+        }
+        if chat_keypad:
+            payload["chat_keypad"] = dataclasses.asdict(chat_keypad)
+        if inline_keypad:
+            payload["inline_keypad"] = dataclasses.asdict(inline_keypad)
+        if reply_to_message_id:
+            payload["reply_to_message_id"] = str(reply_to_message_id)
+        
+        result = await self._make_request("sendFile", payload)
+        result["chat_id"] = chat_id
+        result["client"] = self
+        result["file_id"] = file_id
+        return MessageId(**result)
+    
+    async def send_music(
+        self,
+        chat_id: str,
+        file: Optional[Union[str, Path]] = None,
+        file_id: Optional[str] = None,
+        text: Optional[str] = None,
+        file_name: Optional[str] = None,
+        chat_keypad: Optional[Keypad] = None,
+        inline_keypad: Optional[Keypad] = None,
+        disable_notification: bool = False,
+        reply_to_message_id: Optional[str] = None,
+        chat_keypad_type: ChatKeypadTypeEnum = ChatKeypadTypeEnum.NONE,
+    ) -> MessageId:
+        if file:
+            file_name = file_name or Path(file).name
+            upload_url = await self.request_send_file("File")
+            file_id = await self.upload_file(upload_url, file_name[:-4] + ".ogg", file)
+
+        payload = {
+            "chat_id": chat_id,
+            "file_id": file_id,
+            "text": text,
+            "disable_notification": disable_notification,
+            "chat_keypad_type": chat_keypad_type.value,
+        }
+        if chat_keypad:
+            payload["chat_keypad"] = dataclasses.asdict(chat_keypad)
+        if inline_keypad:
+            payload["inline_keypad"] = dataclasses.asdict(inline_keypad)
+        if reply_to_message_id:
+            payload["reply_to_message_id"] = str(reply_to_message_id)
+
+        result = await self._make_request("sendFile", payload)
+        result["chat_id"] = chat_id
+        result["client"] = self
+        result["file_id"] = file_id
+        return MessageId(**result)
+
+    async def send_voice(
+        self,
+        chat_id: str,
+        file: Optional[Union[str, Path]] = None,
+        file_id: Optional[str] = None,
+        text: Optional[str] = None,
+        file_name: Optional[str] = None,
+        chat_keypad: Optional[Keypad] = None,
+        inline_keypad: Optional[Keypad] = None,
+        disable_notification: bool = False,
+        reply_to_message_id: Optional[str] = None,
+        chat_keypad_type: ChatKeypadTypeEnum = ChatKeypadTypeEnum.NONE,
+    ) -> MessageId:
+        if file:
+            file_name = file_name or Path(file).name
+            upload_url = await self.request_send_file("File")
+            file_id = await self.upload_file(upload_url, file_name[:-4] + ".ogg", file)
+
         payload = {
             "chat_id": chat_id,
             "file_id": file_id,
@@ -255,10 +395,36 @@ class BotClient:
         return result["download_url"]
 
     async def send_poll(
-        self, chat_id: str, question: str, options: List[str]
+        self,
+        chat_id: str,
+        question: str,
+        options: List[str],
+        type: Literal["Regular", "Quiz"] = "Regular",
+        allows_multiple_answers: bool = False,
+        is_anonymous: bool = True,
+        correct_option_index: Optional[int] = None,
+        explanation: Optional[str] = None,
+        reply_to_message_id: Optional[str] = None,
+        disable_notification: bool = False,
+        chat_keypad: Optional[Keypad] = None,
+        chat_keypad_type: ChatKeypadTypeEnum = ChatKeypadTypeEnum.NONE,
+
     ) -> MessageId:
         result = await self._make_request(
-            "sendPoll", {"chat_id": chat_id, "question": question, "options": options}
+            "sendPoll", {
+                "chat_id": chat_id,
+                "question": question,
+                "options": options,
+                "type": type,
+                "allows_multiple_answers": allows_multiple_answers,
+                "is_anonymous": is_anonymous,
+                "correct_option_index": correct_option_index,
+                "explanation": explanation,
+                "reply_to_message_id": reply_to_message_id,
+                "disable_notification": disable_notification,
+                "chat_keypad": dataclasses.asdict(chat_keypad),
+                "chat_keypad_type": chat_keypad_type.value,
+            }
         )
         result["chat_id"] = chat_id
         result["client"] = self
@@ -270,7 +436,6 @@ class BotClient:
         latitude: Union[str, float],
         longitude: Union[str, float],
         chat_keypad: Optional[Keypad] = None,
-        inline_keypad: Optional[Keypad] = None,
         disable_notification: bool = False,
         reply_to_message_id: Optional[str] = None,
         chat_keypad_type: ChatKeypadTypeEnum = ChatKeypadTypeEnum.NONE,
@@ -284,8 +449,6 @@ class BotClient:
         }
         if chat_keypad:
             payload["chat_keypad"] = dataclasses.asdict(chat_keypad)
-        if inline_keypad:
-            payload["inline_keypad"] = dataclasses.asdict(inline_keypad)
         if reply_to_message_id:
             payload["reply_to_message_id"] = str(reply_to_message_id)
         result = await self._make_request("sendLocation", payload)
@@ -300,7 +463,6 @@ class BotClient:
         last_name: str,
         phone_number: str,
         chat_keypad: Optional[Keypad] = None,
-        inline_keypad: Optional[Keypad] = None,
         disable_notification: bool = False,
         reply_to_message_id: Optional[str] = None,
         chat_keypad_type: ChatKeypadTypeEnum = ChatKeypadTypeEnum.NONE,
@@ -316,8 +478,6 @@ class BotClient:
 
         if chat_keypad:
             payload["chat_keypad"] = dataclasses.asdict(chat_keypad)
-        if inline_keypad:
-            payload["inline_keypad"] = dataclasses.asdict(inline_keypad)
         if reply_to_message_id:
             payload["reply_to_message_id"] = str(reply_to_message_id)
 
@@ -397,7 +557,7 @@ class BotClient:
 
         return updates
 
-    async def get_chat(self, chat_id: str) -> Any:
+    async def get_chat(self, chat_id: str) -> Chat:
         result = await self._make_request("getChat", {"chat_id": chat_id})
         return Chat(**result["chat"])
 
@@ -407,7 +567,7 @@ class BotClient:
         message_id: str,
         to_chat_id: str,
         disable_notification: bool = False,
-    ) -> Any:
+    ) -> MessageId:
         result = await self._make_request(
             "forwardMessage",
             {
@@ -421,18 +581,16 @@ class BotClient:
         result["client"] = self
         return MessageId(**result)
 
-    async def edit_message_text(self, chat_id: str, message_id: str, text: str) -> Any:
+    async def edit_message_text(self, chat_id: str, message_id: str, text: str) -> bool:
         result = await self._make_request(
             "editMessageText",
             {"chat_id": chat_id, "message_id": str(message_id), "text": text},
         )
-        if result == {}:
-            return True
-        return False
+        return not bool(result)
 
     async def edit_message_keypad(
         self, chat_id: str, message_id: str, inline_keypad: Keypad
-    ) -> Any:
+    ) -> bool:
         result = await self._make_request(
             "editMessageKeypad",
             {
@@ -441,23 +599,17 @@ class BotClient:
                 "inline_keypad": dataclasses.asdict(inline_keypad),
             },
         )
-        if result == {}:
-            return True
-        return False
+        return not bool(result)
 
     async def delete_message(self, chat_id: str, message_id: str) -> Any:
         result = await self._make_request(
             "deleteMessage", {"chat_id": chat_id, "message_id": str(message_id)}
         )
-        if result == {}:
-            return True
-        return False
+        return not bool(result)
 
-    async def set_commands(self, commands: List[Dict[str, str]]) -> Any:
+    async def set_commands(self, commands: List[Dict[str, str]]) -> bool:
         result = await self._make_request("setCommands", {"bot_commands": commands})
-        if result == {}:
-            return True
-        return False
+        return not bool(result)
 
     async def update_bot_endpoints(self, url: str, endpoint_type: str):
         return await self._make_request(
@@ -475,21 +627,21 @@ class BotClient:
             "message_id": message_id,
             "disable_notification": disable_notification
         }
-        return await self._make_request("pinChatMessage", payload)
+        return not bool(await self._make_request("pinChatMessage", payload))
     
     async def unpin_chat_message(self, chat_id: str, message_id: str):
         payload = {
             "chat_id": chat_id,
             "message_id": message_id
         }
-        return await self._make_request("unpinChatMessage", payload)
+        return not bool(await self._make_request("unpinChatMessage", payload))
     
     async def unpin_all_chat_messages(self, chat_id: str, message_id: str):
         payload = {
             "chat_id": chat_id,
             "message_id": message_id
         }
-        return await self._make_request("unpinAllChatMessages", payload)
+        return not bool(await self._make_request("unpinAllChatMessages", payload))
 
     async def get_chat_administrators(self, chat_id: str):
         return await self._make_request("getChatAdministrators", {"chat_id": chat_id})
@@ -525,14 +677,11 @@ class BotClient:
         chat_id: str,
         keypad_type: ChatKeypadTypeEnum,
         keypad: Optional[Keypad] = None,
-    ) -> Any:
+    ) -> bool:
         payload = {"chat_id": chat_id, "chat_keypad_type": keypad_type.value}
         if keypad:
             payload["chat_keypad"] = dataclasses.asdict(keypad)
-        result = await self._make_request("editChatKeypad", payload)
-        if result == {}:
-            return True
-        return False
+        return not bool(await self._make_request("editChatKeypad", payload))
 
     def on_update(self, *filters: Filter) -> Callable:
         def decorator(handler: Callable) -> Callable:
@@ -546,6 +695,61 @@ class BotClient:
             return handler
 
         return decorator
+    
+    def add_handler(self, handler: Callable, *filters: Filter) -> str:
+        """
+        Register a new handler with given filters.
+
+        Args:
+            handler (Callable): The function to handle updates.
+            *filters (Filter): One or more filters that must pass.
+
+        Returns:
+            str: Unique handler key (can be used to remove handler later).
+        
+        Example:
+            key = bot.add_handler(my_handler, TextFilter("hello"))
+        """
+        handler_key = str(uuid.uuid4())
+        if handler_key not in self.handlers:
+            self.handlers[handler_key] = []
+        self.handlers[handler_key].append((filters, handler))
+        logger.info(
+            f"Handler {handler.__name__} added with key {handler_key}, filters: {filters}"
+        )
+        return handler_key
+
+    def remove_handler(self, handler_key: str, handler: Optional[Callable] = None) -> bool:
+        """
+        Remove a registered handler.
+
+        Args:
+            handler_key (str): The unique key returned by add_handler/on_update.
+            handler (Optional[Callable]): Specific handler function to remove.
+                                          If None, removes all handlers under the key.
+
+        Returns:
+            bool: True if something was removed, False otherwise.
+
+        Example:
+            bot.remove_handler(key)
+        """
+        if handler_key not in self.handlers:
+            return False
+
+        if handler:
+            before = len(self.handlers[handler_key])
+            self.handlers[handler_key] = [
+                (filters, h) for filters, h in self.handlers[handler_key] if h != handler
+            ]
+            after = len(self.handlers[handler_key])
+            removed = before != after
+            if not self.handlers[handler_key]:  # clean up empty key
+                del self.handlers[handler_key]
+            return removed
+        else:
+            del self.handlers[handler_key]
+            return True
 
     def _parse_update(self, item: Dict) -> Optional[Union[Update, InlineMessage]]:
         update_type = item.get("type")
@@ -619,64 +823,35 @@ class BotClient:
             return None
 
     async def process_update(self, update: Union[Update, InlineMessage]):
-        update_type = (
-            "InlineMessage" if isinstance(update, InlineMessage) else update.type
-        )
+        async def run_middlewares(index: int):
+            if index < len(self.middlewares):
+                mw = self.middlewares[index]
+                if inspect.iscoroutinefunction(mw):
+                    await mw(self, update, lambda: run_middlewares(index + 1))
+                else:
+                    mw(self, update, lambda: asyncio.create_task(run_middlewares(index + 1)))
+            else:
+                await self._dispatch_update(update)
+
+        await run_middlewares(0)
+
+    async def _dispatch_update(self, update: Union[Update, InlineMessage]):
+        update_type = "InlineMessage" if isinstance(update, InlineMessage) else update.type
         message_id = self._extract_message_id(update)
 
-        # Skip processed messages check for InlineMessage to avoid issues with repeated message_id
-        if (
-            isinstance(update, Update)
-            and message_id
-            and message_id in self.processed_messages
-        ):
-            logger.debug(f"Skipping processed update ({update_type}): {message_id}")
+        if isinstance(update, Update) and message_id and message_id in self.processed_messages:
             return
-
         if message_id and isinstance(update, Update):
             self.processed_messages.append(message_id)
 
-        logger.info(
-            f"Processing update: {update_type}, handlers available: {len(self.handlers)}"
-        )
-        handled = False
         for filter_key, handler_list in self.handlers.items():
-            logger.info(
-                f"Checking handler list {filter_key} with {len(handler_list)} handlers"
-            )
             for filters, handler in handler_list:
-                logger.info(
-                    f"Checking filters for handler {handler.__name__}: {filters}"
-                )
                 if await self._filters_pass(update, filters):
-                    try:
-                        logger.info(
-                            f"Executing handler {handler.__name__} for update {update_type}"
-                        )
-                        if asyncio.iscoroutinefunction(handler):
-                            asyncio.create_task(handler(self, update))
-                        else:
-                            threading.Thread(
-                                target=handler,
-                                args=(
-                                    self,
-                                    update,
-                                ),
-                            ).start()
-                        handled = True
-                        return
-                    except Exception as e:
-                        logger.exception(
-                            f"Error in handler {handler.__name__}: {str(e)}"
-                        )
-                else:
-                    logger.info(
-                        f"Filters {filters} failed for handler {handler.__name__} on update {update_type}"
-                    )
-        if not handled:
-            logger.warning(
-                f"No handlers matched for update {update_type} with data: {update}"
-            )
+                    if asyncio.iscoroutinefunction(handler):
+                        asyncio.create_task(handler(self, update))
+                    else:
+                        threading.Thread(target=handler, args=(self, update)).start()
+                    return
 
     def _extract_message_id(
         self, update: Union[Update, InlineMessage]
@@ -695,19 +870,14 @@ class BotClient:
     async def _filters_pass(
         self, update: Union[Update, InlineMessage], filters: Tuple[Filter, ...]
     ) -> bool:
-        update_type = (
-            "InlineMessage" if isinstance(update, InlineMessage) else update.type
-        )
+        update_type = ("InlineMessage" if isinstance(update, InlineMessage) else update.type)
 
         for f in filters:
-            # اگر کلاس هست (و نه نمونه)، نمونه‌سازی کن
             if isinstance(f, type):
                 f = f()
 
             result = await f.check(update)
-            logger.info(
-                f"Filter {f.__class__.__name__} check for update {update_type}: {result}"
-            )
+            logger.info(f"Filter {f.__class__.__name__} check for update {update_type}: {result}")
             if not result:
                 return False
 
@@ -770,7 +940,7 @@ class BotClient:
         webhook_url: Optional[str] = None,
         path: Optional[str] = "/webhook",
         host: str = "0.0.0.0",
-        port: int = 8080,
+        port: int = 8080
     ):
         self.use_webhook = bool(webhook_url)
         await self.start()
@@ -822,6 +992,7 @@ class BotClient:
         progress: Optional[Callable[[int, int], None]] = None,
         chunk_size: int = 65536,
         as_bytes: bool = False,
+        timeout: Optional[Union[int, float]] = 20.0
     ) -> Union[str, bytes, None]:
         """
         Downloads a file by file_id.
@@ -845,13 +1016,14 @@ class BotClient:
         if not download_url:
             raise ValueError(f"Invalid download file url: {download_url}")
 
-        async with aiohttp.ClientSession() as session:
-            async with session.post(download_url) as response:
+        async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(total=timeout)) as session:
+            async with session.get(download_url) as response:
                 if response.status != 200:
                     raise Exception(f"Failed to download file: {response.status}")
 
                 content_type = response.headers.get("Content-Type", "")
-                ext = get_extension(content_type)
+                ext = mimetypes.guess_extension(content_type)
+                ext = ext if ext else ""
                 total_size = int(response.headers.get("Content-Length", 0))
                 downloaded = 0
 
