@@ -429,12 +429,11 @@ class Update(DictLike):
     ) -> MessageId:
         if not self.client:
             raise ValueError("Client not set for Update")
-        return await self.client.send_file(
+        return await self.client.send_voice(
             chat_id=chat_id or self.chat_id,
             file=file,
             file_id=file_id,
             text=text,
-            type="Image",
             chat_keypad=chat_keypad,
             inline_keypad=inline_keypad,
             disable_notification=disable_notification,
@@ -457,12 +456,11 @@ class Update(DictLike):
     ) -> MessageId:
         if not self.client:
             raise ValueError("Client not set for Update")
-        return await self.client.send_file(
+        return await self.client.send_music(
             chat_id=chat_id or self.chat_id,
             file=file,
             file_id=file_id,
             text=text,
-            type="Music",
             chat_keypad=chat_keypad,
             inline_keypad=inline_keypad,
             disable_notification=disable_notification,
