@@ -1,4 +1,3 @@
-
 import asyncio
 import functools
 import inspect
@@ -10,6 +9,7 @@ from rubpy.rubino import Rubino
 from rubpy.bot import BotClient
 from rubpy.bot import models
 
+
 def async_to_sync(obj, name):
     """
     Wrap an asynchronous function or asynchronous generator method
@@ -36,7 +36,9 @@ def async_to_sync(obj, name):
             if is_main_thread:
                 item, done = loop.run_until_complete(anext(agen))
             else:
-                item, done = asyncio.run_coroutine_threadsafe(anext(agen), loop).result()
+                item, done = asyncio.run_coroutine_threadsafe(
+                    anext(agen), loop
+                ).result()
 
             if done:
                 break
@@ -53,7 +55,10 @@ def async_to_sync(obj, name):
             loop = asyncio.new_event_loop()
             asyncio.set_event_loop(loop)
 
-        if threading.current_thread() is threading.main_thread() or not main_loop.is_running():
+        if (
+            threading.current_thread() is threading.main_thread()
+            or not main_loop.is_running()
+        ):
             if loop.is_running():
                 return coroutine
             else:
@@ -65,12 +70,17 @@ def async_to_sync(obj, name):
         else:
             if inspect.iscoroutine(coroutine):
                 if loop.is_running():
+
                     async def coro_wrapper():
-                        return await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(coroutine, main_loop))
+                        return await asyncio.wrap_future(
+                            asyncio.run_coroutine_threadsafe(coroutine, main_loop)
+                        )
 
                     return coro_wrapper()
                 else:
-                    return asyncio.run_coroutine_threadsafe(coroutine, main_loop).result()
+                    return asyncio.run_coroutine_threadsafe(
+                        coroutine, main_loop
+                    ).result()
 
             if inspect.isasyncgen(coroutine):
                 if loop.is_running():
@@ -80,6 +90,7 @@ def async_to_sync(obj, name):
 
     setattr(obj, name, async_to_sync_wrap)
 
+
 def wrap_methods(source):
     """
     Wrap asynchronous methods in a class to make them synchronous.
@@ -90,9 +101,12 @@ def wrap_methods(source):
     for name in dir(source):
         method = getattr(source, name)
 
-        if not name.startswith("_") and (inspect.iscoroutinefunction(method) or inspect.isasyncgenfunction(method)):
+        if not name.startswith("_") and (
+            inspect.iscoroutinefunction(method) or inspect.isasyncgenfunction(method)
+        ):
             async_to_sync(source, name)
 
+
 def wrap_types_methods():
     """
     Wrap asynchronous methods in types' classes to make them synchronous.
@@ -103,6 +117,7 @@ def wrap_types_methods():
         if inspect.isclass(cls):
             wrap_methods(cls)
 
+
 def wrap_models_methods():
     """
     Wrap asynchronous methods in models' classes to make them synchronous.
@@ -113,6 +128,7 @@ def wrap_models_methods():
         if inspect.isclass(cls):
             wrap_methods(cls)
 
+
 # Wrap all relevant methods in the Client's Methods class
 wrap_methods(Methods)
 
